@@ -12,6 +12,7 @@ import (
 
 	corev1 "k8s.io/api/core/v1"
 	corev1client "k8s.io/client-go/kubernetes/typed/core/v1"
+	corev1lister "k8s.io/client-go/listers/core/v1"
 
 	kerrors "k8s.io/apimachinery/pkg/api/errors"
 	kapis "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -20,7 +21,6 @@ import (
 	"k8s.io/apimachinery/pkg/util/wait"
 
 	restclient "k8s.io/client-go/rest"
-	controllercache "k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/util/retry"
 
 	imagev1 "github.com/openshift/api/image/v1"
@@ -28,7 +28,10 @@ import (
 
 	configv1client "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
 	imagev1client "github.com/openshift/client-go/image/clientset/versioned/typed/image/v1"
+	imagev1lister "github.com/openshift/client-go/image/listers/image/v1"
 	templatev1client "github.com/openshift/client-go/template/clientset/versioned/typed/template/v1"
+	templatev1lister "github.com/openshift/client-go/template/listers/template/v1"
+	configv1lister "github.com/openshift/cluster-samples-operator/pkg/generated/listers/samples/v1"
 
 	operatorsv1api "github.com/openshift/api/operator/v1"
 	v1 "github.com/openshift/cluster-samples-operator/pkg/apis/samples/v1"
@@ -46,7 +49,12 @@ const (
 	skippedtempskey      = "keyForSkippedTemplatesField"
 )
 
-func NewSamplesOperatorHandler(kubeconfig *restclient.Config, cachestore controllercache.Store) (*Handler, error) {
+func NewSamplesOperatorHandler(kubeconfig *restclient.Config,
+	configLister configv1lister.ConfigLister,
+	streamLister imagev1lister.ImageStreamNamespaceLister,
+	templateLister templatev1lister.TemplateNamespaceLister,
+	openshiftNamespaceSecretLister corev1lister.SecretNamespaceLister,
+	configNamespaceSecretLister corev1lister.SecretNamespaceLister) (*Handler, error) {
 	h := &Handler{}
 
 	h.initter = &defaultInClusterInitter{}
@@ -58,19 +66,23 @@ func NewSamplesOperatorHandler(kubeconfig *restclient.Config, cachestore control
 		return nil, err
 	}
 	crdWrapper.client = client.Configs()
-	crdWrapper.store = cachestore
+	crdWrapper.lister = configLister
 
 	h.crdwrapper = crdWrapper
 
-	h.crdstore = cachestore
+	h.crdlister = configLister
+	h.streamlister = streamLister
+	h.tplstore = templateLister
+	h.opshiftsecretlister = openshiftNamespaceSecretLister
+	h.cfgsecretlister = configNamespaceSecretLister
 
 	h.Fileimagegetter = &DefaultImageStreamFromFileGetter{}
 	h.Filetemplategetter = &DefaultTemplateFromFileGetter{}
 	h.Filefinder = &DefaultResourceFileLister{}
 
-	h.imageclientwrapper = &defaultImageStreamClientWrapper{h: h}
-	h.templateclientwrapper = &defaultTemplateClientWrapper{h: h}
-	h.secretclientwrapper = &defaultSecretClientWrapper{coreclient: h.coreclient}
+	h.imageclientwrapper = &defaultImageStreamClientWrapper{h: h, lister: streamLister}
+	h.templateclientwrapper = &defaultTemplateClientWrapper{h: h, lister: templateLister}
+	h.secretclientwrapper = &defaultSecretClientWrapper{coreclient: h.coreclient, opnshftlister: openshiftNamespaceSecretLister, cfglister: configNamespaceSecretLister}
 	h.cvowrapper = operatorstatus.NewClusterOperatorHandler(h.configclient)
 
 	h.skippedImagestreams = make(map[string]bool)
@@ -103,7 +115,11 @@ type Handler struct {
 	templateclientwrapper TemplateClientWrapper
 	secretclientwrapper   SecretClientWrapper
 
-	crdstore controllercache.Store
+	crdlister           configv1lister.ConfigLister
+	streamlister        imagev1lister.ImageStreamNamespaceLister
+	tplstore            templatev1lister.TemplateNamespaceLister
+	opshiftsecretlister corev1lister.SecretNamespaceLister
+	cfgsecretlister     corev1lister.SecretNamespaceLister
 
 	Fileimagegetter    ImageStreamFromFileGetter
 	Filetemplategetter TemplateFromFileGetter
@@ -117,6 +133,7 @@ type Handler struct {
 
 	mapsMutex sync.Mutex
 
+	upsertInProgress bool
 	secretRetryCount int8
 	version          string
 }
@@ -130,17 +147,24 @@ type Handler struct {
 func (h *Handler) prepSamplesWatchEvent(kind, name string, annotations map[string]string, deleted bool) (*v1.Config, string, bool, error) {
 	cfg, err := h.crdwrapper.Get(v1.ConfigName)
 	if cfg == nil || err != nil {
+		// if not found, then this also would mean a deletion event
+		if kerrors.IsNotFound(err) {
+			logrus.Printf("Received watch event %s but not upserting since deletion of the Config is in progress", kind+"/"+name)
+			return nil, "", false, nil
+		}
 		logrus.Printf("Received watch event %s but not upserting since not have the Config yet: %#v %#v", kind+"/"+name, err, cfg)
 		return nil, "", false, err
 	}
 
+	if cfg.DeletionTimestamp != nil {
+		// we do no return the cfg in this case because we do not want to bother with any progress tracking
+		logrus.Printf("Received watch event %s but not upserting since deletion of the Config is in progress", kind+"/"+name)
+		// note, the imagestream watch cache gets cleared once the deletion/finalizer processing commences
+		return nil, "", false, nil
+	}
+
 	if cfg.ConditionFalse(v1.ImageChangesInProgress) {
 		// we do no return the cfg in these cases because we do not want to bother with any progress tracking
-
-		if cfg.DeletionTimestamp != nil {
-			logrus.Printf("Received watch event %s but not upserting since deletion of the Config is in progress and image changes are not in progress", kind+"/"+name)
-			return nil, "", false, nil
-		}
 		switch cfg.Spec.ManagementState {
 		case operatorsv1api.Removed:
 			logrus.Debugf("Not upserting %s/%s event because operator is in removed state and image changes are not in progress", kind, name)
@@ -216,8 +240,10 @@ func (h *Handler) GoodConditionUpdate(cfg *v1.Config, newStatus corev1.Condition
 	if condition.Status != newStatus {
 		now := kapis.Now()
 		condition.LastUpdateTime = now
+		if condition.Status != newStatus {
+			condition.LastTransitionTime = now
+		}
 		condition.Status = newStatus
-		condition.LastTransitionTime = now
 		condition.Message = ""
 		condition.Reason = ""
 		cfg.ConditionUpdate(condition)
@@ -342,7 +368,7 @@ func (h *Handler) CleanUpOpenshiftNamespaceOnDelete(cfg *v1.Config) error {
 
 	iopts := metav1.ListOptions{LabelSelector: v1.SamplesManagedLabel + "=true"}
 
-	streamList, err := h.imageclientwrapper.List("openshift", iopts)
+	streamList, err := h.imageclientwrapper.List(iopts)
 	if err != nil && !kerrors.IsNotFound(err) {
 		logrus.Warnf("Problem listing openshift imagestreams on Config delete: %#v", err)
 		return err
@@ -355,7 +381,7 @@ func (h *Handler) CleanUpOpenshiftNamespaceOnDelete(cfg *v1.Config) error {
 				if !ok || strings.TrimSpace(manage) != "true" {
 					continue
 				}
-				err = h.imageclientwrapper.Delete("openshift", stream.Name, &metav1.DeleteOptions{})
+				err = h.imageclientwrapper.Delete(stream.Name, &metav1.DeleteOptions{})
 				if err != nil && !kerrors.IsNotFound(err) {
 					logrus.Warnf("Problem deleting openshift imagestream %s on Config delete: %#v", stream.Name, err)
 					return err
@@ -367,7 +393,7 @@ func (h *Handler) CleanUpOpenshiftNamespaceOnDelete(cfg *v1.Config) error {
 
 	cache.ClearUpsertsCache()
 
-	tempList, err := h.templateclientwrapper.List("openshift", iopts)
+	tempList, err := h.templateclientwrapper.List(iopts)
 	if err != nil && !kerrors.IsNotFound(err) {
 		logrus.Warnf("Problem listing openshift templates on Config delete: %#v", err)
 		return err
@@ -380,7 +406,7 @@ func (h *Handler) CleanUpOpenshiftNamespaceOnDelete(cfg *v1.Config) error {
 				if !ok || strings.TrimSpace(manage) != "true" {
 					continue
 				}
-				err = h.templateclientwrapper.Delete("openshift", temp.Name, &metav1.DeleteOptions{})
+				err = h.templateclientwrapper.Delete(temp.Name, &metav1.DeleteOptions{})
 				if err != nil && !kerrors.IsNotFound(err) {
 					logrus.Warnf("Problem deleting openshift template %s on Config delete: %#v", temp.Name, err)
 					return err
@@ -446,16 +472,26 @@ func (h *Handler) Handle(event v1.Event) error {
 		// and event delete flag true
 		if event.Deleted {
 			logrus.Info("A previous delete attempt has been successfully completed")
+			h.cvowrapper.UpdateOperatorStatus(cfg, true)
 			return nil
 		}
 		if cfg.DeletionTimestamp != nil {
+			h.cvowrapper.UpdateOperatorStatus(cfg, true)
 			// before we kick off the delete cycle though, we make sure a prior creation
 			// cycle is not still in progress, because we don't want the create adding back
-			// in things we just deleted ... if an upsert is still in progress, return nil
-			if cfg.ConditionTrue(v1.ImageChangesInProgress) {
-				return nil
+			// in things we just deleted ... if an upsert is still in progress, return an error;
+			// the creation loop checks for deletion timestamp and aborts when it sees it;
+			// but we don't use in progress condition here, as the upsert cycle might also be complete;
+			// but ImageInProess is still true, so we use a local variable which is only set while upserts are happening; otherwise
+			// here, we get started with the delete, which will ultimately reset the conditions
+			// and samples, when it is false again, regardless of whether the imagestream imports are done
+			if h.upsertInProgress {
+				return fmt.Errorf("A delete attempt has come in while creating samples; initiating retry; creation loop should abort soon")
 			}
 
+			// nuke any registered upserts
+			cache.ClearUpsertsCache()
+
 			if h.NeedsFinalizing(cfg) {
 				// so we initiate the delete and set exists to false first, where if we get
 				// conflicts because of start up imagestream events, the retry should work
@@ -474,8 +510,9 @@ func (h *Handler) Handle(event v1.Event) error {
 					return err
 				}
 				h.GoodConditionUpdate(cfg, corev1.ConditionFalse, v1.SamplesExist)
-				logrus.Printf("CRDUPDATE exist false update")
-				err = h.crdwrapper.UpdateStatus(cfg)
+				dbg := "exist false update"
+				logrus.Printf("CRDUPDATE %s", dbg)
+				err = h.crdwrapper.UpdateStatus(cfg, dbg)
 				if err != nil {
 					logrus.Printf("error on Config update after setting exists condition to false (returning error to retry): %v", err)
 					return err
@@ -483,7 +520,8 @@ func (h *Handler) Handle(event v1.Event) error {
 			} else {
 				logrus.Println("Initiating finalizer processing for a SampleResource delete attempt")
 				h.RemoveFinalizer(cfg)
-				logrus.Printf("CRDUPDATE remove finalizer update")
+				dbg := "remove finalizer update"
+				logrus.Printf("CRDUPDATE %s", dbg)
 				// not updating the status, but the metadata annotation
 				err := h.crdwrapper.Update(cfg)
 				if err != nil {
@@ -497,47 +535,56 @@ func (h *Handler) Handle(event v1.Event) error {
 			return nil
 		}
 
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
 		if cfg.ConditionUnknown(v1.ImportCredentialsExist) {
 			// retry the default cred copy if it failed previously
 			err := h.copyDefaultClusterPullSecret(nil)
 			if err == nil {
+				cfg = h.refetchCfgMinimizeConflicts(cfg)
 				h.GoodConditionUpdate(cfg, corev1.ConditionTrue, v1.ImportCredentialsExist)
-				logrus.Println("CRDUPDATE cleared import cred unknown")
-				return h.crdwrapper.UpdateStatus(cfg)
+				dbg := "cleared import cred unknown"
+				logrus.Printf("CRDUPDATE %s", dbg)
+				return h.crdwrapper.UpdateStatus(cfg, dbg)
 			}
 		}
 
 		// Every time we see a change to the Config object, update the ClusterOperator status
 		// based on the current conditions of the Config.
-		err := h.cvowrapper.UpdateOperatorStatus(cfg)
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
+		err := h.cvowrapper.UpdateOperatorStatus(cfg, false)
 		if err != nil {
 			logrus.Errorf("error updating cluster operator status: %v", err)
 			return err
 		}
 
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
 		doit, cfgUpdate, err := h.ProcessManagementField(cfg)
 		if !doit || err != nil {
 			if err != nil || cfgUpdate {
 				// flush status update
-				logrus.Printf("CRDUPDATE process mgmt update")
-				h.crdwrapper.UpdateStatus(cfg)
+				dbg := "process mgmt update"
+				logrus.Printf("CRDUPDATE %s", dbg)
+				return h.crdwrapper.UpdateStatus(cfg, dbg)
 			}
 			return err
 		}
 
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
 		existingValidStatus := cfg.Condition(v1.ConfigurationValid).Status
 		err = h.SpecValidation(cfg)
 		if err != nil {
 			// flush status update
-			logrus.Printf("CRDUPDATE bad spec validation update")
+			dbg := "bad spec validation update"
+			logrus.Printf("CRDUPDATE %s", dbg)
 			// only retry on error updating the Config; do not return
 			// the error from SpecValidation which denotes a bad config
-			return h.crdwrapper.UpdateStatus(cfg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 		// if a bad config was corrected, update and return
 		if existingValidStatus != cfg.Condition(v1.ConfigurationValid).Status {
-			logrus.Printf("CRDUPDATE spec corrected")
-			return h.crdwrapper.UpdateStatus(cfg)
+			dbg := "spec corrected"
+			logrus.Printf("CRDUPDATE %s", dbg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 
 		h.buildSkipFilters(cfg)
@@ -548,6 +595,7 @@ func (h *Handler) Handle(event v1.Event) error {
 		unskippedStreams := map[string]bool{}
 		unskippedTemplates := map[string]bool{}
 		if cfg.Spec.ManagementState == cfg.Status.ManagementState {
+			cfg = h.refetchCfgMinimizeConflicts(cfg)
 			configChanged, configChangeRequiresUpsert, configChangeRequiresImportErrorUpdate, registryChanged, unskippedStreams, unskippedTemplates = h.VariableConfigChanged(cfg)
 			logrus.Debugf("config changed %v upsert needed %v import error upd needed %v exists/true %v progressing/false %v op version %s status version %s",
 				configChanged,
@@ -568,15 +616,17 @@ func (h *Handler) Handle(event v1.Event) error {
 				// once the status version is in sync, we can turn off the migration condition
 				if cfg.ConditionTrue(v1.MigrationInProgress) {
 					h.GoodConditionUpdate(cfg, corev1.ConditionFalse, v1.MigrationInProgress)
-					logrus.Println("CRDUPDATE turn migration off")
-					return h.crdwrapper.UpdateStatus(cfg)
+					dbg := " turn migration off"
+					logrus.Printf("CRDUPDATE %s", dbg)
+					return h.crdwrapper.UpdateStatus(cfg, dbg)
 				}
 
 				// in case this is a bring up after initial install, we take a pass
 				// and see if any samples were deleted while samples operator was down
 				h.buildFileMaps(cfg, false)
 				// passing in false means if the samples is present, we leave it alone
-				return h.createSamples(cfg, false, registryChanged, unskippedStreams, unskippedTemplates)
+				_, err = h.createSamples(cfg, false, registryChanged, unskippedStreams, unskippedTemplates)
+				return err
 			}
 			// if config changed requiring an upsert, but a prior config action is still in progress,
 			// reset in progress to false and return; the next event should drive the actual
@@ -585,8 +635,9 @@ func (h *Handler) Handle(event v1.Event) error {
 			if configChangeRequiresUpsert &&
 				cfg.ConditionTrue(v1.ImageChangesInProgress) {
 				h.GoodConditionUpdate(cfg, corev1.ConditionFalse, v1.ImageChangesInProgress)
-				logrus.Printf("CRDUPDATE change in progress from true to false for config change")
-				return h.crdwrapper.UpdateStatus(cfg)
+				dbg := "change in progress from true to false for config change"
+				logrus.Printf("CRDUPDATE %s", dbg)
+				return h.crdwrapper.UpdateStatus(cfg, dbg)
 			}
 		}
 
@@ -604,29 +655,34 @@ func (h *Handler) Handle(event v1.Event) error {
 		// if trying to do rhel to the default registry.redhat.io registry requires the secret
 		// be in place since registry.redhat.io requires auth to pull; if it is not ready
 		// error state will be logged by WaitingForCredential
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
 		stillWaitingForSecret, callSDKToUpdate := h.WaitingForCredential(cfg)
 		if callSDKToUpdate {
 			// flush status update ... the only error generated by WaitingForCredential, not
 			// by api obj access
-			logrus.Println("CRDUPDATE Config update ignored since need the RHEL credential")
+			dbg := "Config update ignored since need the RHEL credential"
+			logrus.Printf("CRDUPDATE %s", dbg)
 			// if update to set import cred condition to false fails, return that error
 			// to requeue
-			return h.crdwrapper.UpdateStatus(cfg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 		if stillWaitingForSecret {
 			// means we previously udpated cfg but nothing has changed wrt the secret's presence
 			return nil
 		}
 
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
 		if cfg.ConditionFalse(v1.MigrationInProgress) &&
 			len(cfg.Status.Version) > 0 &&
 			h.version != cfg.Status.Version {
 			logrus.Printf("Undergoing migration from %s to %s", cfg.Status.Version, h.version)
 			h.GoodConditionUpdate(cfg, corev1.ConditionTrue, v1.MigrationInProgress)
-			logrus.Println("CRDUPDATE turn migration on")
-			return h.crdwrapper.UpdateStatus(cfg)
+			dbg := "turn migration on"
+			logrus.Printf("CRDUPDATE %s", dbg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
 		if !configChanged &&
 			cfg.ConditionTrue(v1.SamplesExist) &&
 			cfg.ConditionFalse(v1.ImageChangesInProgress) &&
@@ -637,8 +693,9 @@ func (h *Handler) Handle(event v1.Event) error {
 			}
 			cfg.Status.Version = h.version
 			logrus.Printf("The samples are now at version %s", cfg.Status.Version)
-			logrus.Println("CRDUPDATE upd status version")
-			return h.crdwrapper.UpdateStatus(cfg)
+			dbg := "upd status version"
+			logrus.Printf("CRDUPDATE %s", dbg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 			/*if cfg.ConditionFalse(v1.ImportImageErrorsExist) {
 				cfg.Status.Version = h.version
 				logrus.Printf("The samples are now at version %s", cfg.Status.Version)
@@ -669,13 +726,15 @@ func (h *Handler) Handle(event v1.Event) error {
 		// this boolean is driven by VariableConfigChanged based on comparing spec/status skip lists and
 		// cross referencing with any image import errors
 		if configChangeRequiresImportErrorUpdate && !configChangeRequiresUpsert {
-			logrus.Printf("CRDUPDATE config change did not require upsert but did change import errors")
-			return h.crdwrapper.UpdateStatus(cfg)
+			dbg := "config change did not require upsert but did change import errors"
+			logrus.Printf("CRDUPDATE %s", dbg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 
 		if configChanged && !configChangeRequiresUpsert && cfg.ConditionTrue(v1.SamplesExist) {
-			logrus.Printf("CRDUPDATE bypassing upserts for non invasive config change after initial create")
-			return h.crdwrapper.UpdateStatus(cfg)
+			dbg := "bypassing upserts for non invasive config change after initial create"
+			logrus.Printf("CRDUPDATE %s", dbg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 
 		if !cfg.ConditionTrue(v1.ImageChangesInProgress) {
@@ -686,17 +745,36 @@ func (h *Handler) Handle(event v1.Event) error {
 				return err
 			}
 
-			err = h.createSamples(cfg, true, registryChanged, unskippedStreams, unskippedTemplates)
+			h.upsertInProgress = true
+			turnFlagOff := func(h *Handler) { h.upsertInProgress = false }
+			defer turnFlagOff(h)
+			abortForDelete, err := h.createSamples(cfg, true, registryChanged, unskippedStreams, unskippedTemplates)
+			// we prioritize enabling delete vs. any error processing from createSamples (though at the moment that
+			// method only returns nil error when it returns true for abortForDelete) as a subsequent delete's processing will
+			// immediately remove the cfg obj and cluster operator object that we just posted some error notice in
+			if abortForDelete {
+				// a delete has been initiated; let's revert in progress to false and allow the delete to complete,
+				// including its removal of any sample we might have upserted with the above createSamples call
+				h.GoodConditionUpdate(h.refetchCfgMinimizeConflicts(cfg), corev1.ConditionFalse, v1.ImageChangesInProgress)
+				// note, the imagestream watch cache gets cleared once the deletion/finalizer processing commences
+				dbg := "progressing false because delete has arrived"
+				logrus.Printf("CRDUPDATE %s", dbg)
+				return h.crdwrapper.UpdateStatus(cfg, dbg)
+			}
+
 			if err != nil {
+				cfg = h.refetchCfgMinimizeConflicts(cfg)
 				h.processError(cfg, v1.ImageChangesInProgress, corev1.ConditionUnknown, err, "error creating samples: %v")
-				logrus.Printf("CRDUPDATE setting in progress to unknown")
-				e := h.crdwrapper.UpdateStatus(cfg)
+				dbg := "setting in progress to unknown"
+				logrus.Printf("CRDUPDATE %s", dbg)
+				e := h.crdwrapper.UpdateStatus(cfg, dbg)
 				if e != nil {
 					return e
 				}
 				return err
 			}
 			now := kapis.Now()
+			cfg = h.refetchCfgMinimizeConflicts(cfg)
 			progressing := cfg.Condition(v1.ImageChangesInProgress)
 			progressing.LastUpdateTime = now
 			progressing.LastTransitionTime = now
@@ -721,18 +799,17 @@ func (h *Handler) Handle(event v1.Event) error {
 			// of the "make a change" flow in our state machine
 			cfg = h.initConditions(cfg)
 
-			logrus.Printf("CRDUPDATE progressing true update")
-			err = h.crdwrapper.UpdateStatus(cfg)
-			if err != nil {
-				return err
-			}
-			return nil
+			dbg := "progressing true update"
+			logrus.Printf("CRDUPDATE %s", dbg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 
 		if !cfg.ConditionTrue(v1.SamplesExist) {
+			cfg = h.refetchCfgMinimizeConflicts(cfg)
 			h.GoodConditionUpdate(cfg, corev1.ConditionTrue, v1.SamplesExist)
-			logrus.Printf("CRDUPDATE exist true update")
-			return h.crdwrapper.UpdateStatus(cfg)
+			dbg := "exist true update"
+			logrus.Printf("CRDUPDATE %s", dbg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 
 		// it is possible that all the cached imagestream events show that
@@ -743,13 +820,15 @@ func (h *Handler) Handle(event v1.Event) error {
 			keysToClear := []string{}
 			anyChange := false
 			ac := false
+			cfg = h.refetchCfgMinimizeConflicts(cfg)
 			for key, is := range cache.GetUpsertImageStreams() {
 				if is == nil {
 					// never got update, refetch
 					var e error
-					is, e = h.imageclientwrapper.Get("openshift", key, metav1.GetOptions{})
+					is, e = h.imageclientwrapper.Get(key)
 					if e != nil {
 						keysToClear = append(keysToClear, key)
+						anyChange = true
 						continue
 					}
 				}
@@ -766,8 +845,9 @@ func (h *Handler) Handle(event v1.Event) error {
 				logrus.Println("The last in progress imagestream has completed (config event loop)")
 			}
 			if anyChange {
-				logrus.Printf("CRDUPDATE updating in progress after examining cached imagestream events")
-				err = h.crdwrapper.UpdateStatus(cfg)
+				dbg := "updating in progress after examining cached imagestream events"
+				logrus.Printf("CRDUPDATE %s", dbg)
+				err = h.crdwrapper.UpdateStatus(cfg, dbg)
 				if err == nil && cfg.ConditionFalse(v1.ImageChangesInProgress) {
 					// only clear out cache if we got the update through
 					cache.ClearUpsertsCache()
@@ -785,12 +865,12 @@ func (h *Handler) setSampleManagedLabelToFalse(kind, name string) error {
 	case "imagestream":
 		var stream *imagev1.ImageStream
 		err = retry.RetryOnConflict(retry.DefaultBackoff, func() error {
-			stream, err = h.imageclientwrapper.Get("openshift", name, metav1.GetOptions{})
+			stream, err = h.imageclientwrapper.Get(name)
 			if err == nil && stream != nil && stream.Labels != nil {
 				label, _ := stream.Labels[v1.SamplesManagedLabel]
 				if label == "true" {
 					stream.Labels[v1.SamplesManagedLabel] = "false"
-					_, err = h.imageclientwrapper.Update("openshift", stream)
+					_, err = h.imageclientwrapper.Update(stream)
 				}
 			}
 			return err
@@ -798,12 +878,12 @@ func (h *Handler) setSampleManagedLabelToFalse(kind, name string) error {
 	case "template":
 		var tpl *templatev1.Template
 		err = retry.RetryOnConflict(retry.DefaultBackoff, func() error {
-			tpl, err = h.templateclientwrapper.Get("openshift", name, metav1.GetOptions{})
+			tpl, err = h.templateclientwrapper.Get(name)
 			if err == nil && tpl != nil && tpl.Labels != nil {
 				label, _ := tpl.Labels[v1.SamplesManagedLabel]
 				if label == "true" {
 					tpl.Labels[v1.SamplesManagedLabel] = "false"
-					_, err = h.templateclientwrapper.Update("openshift", tpl)
+					_, err = h.templateclientwrapper.Update(tpl)
 				}
 			}
 			return err
@@ -812,14 +892,30 @@ func (h *Handler) setSampleManagedLabelToFalse(kind, name string) error {
 	return nil
 }
 
-func (h *Handler) createSamples(cfg *v1.Config, updateIfPresent, registryChanged bool, unskippedStreams, unskippedTemplates map[string]bool) error {
+// abortForDelete is used in various portions of the control flow to abort upsert or watch processing
+// if a deletion has been initiated and we want to process the config object's finalizer
+func (h *Handler) abortForDelete(cfg *v1.Config) bool {
+	// reminder refetchCfgMinimizeConflicts accesses the lister/watch cache and does not perform API calls
+	cfg = h.refetchCfgMinimizeConflicts(cfg)
+	if cfg.DeletionTimestamp != nil {
+		return true
+	}
+	return false
+}
+
+// rc:  error - any api errors during upserts
+// rc:  bool - if we abort because we detected a delete
+func (h *Handler) createSamples(cfg *v1.Config, updateIfPresent, registryChanged bool, unskippedStreams, unskippedTemplates map[string]bool) (bool, error) {
 	// first, got through the list and prime our upsert cache
 	// prior to any actual upserts
 	imagestreams := []*imagev1.ImageStream{}
 	for _, fileName := range h.imagestreamFile {
+		if h.abortForDelete(cfg) {
+			return true, nil
+		}
 		imagestream, err := h.Fileimagegetter.Get(fileName)
 		if err != nil {
-			return err
+			return false, err
 		}
 
 		// if unskippedStreams has >0 entries, then we are down this path to only upsert the streams
@@ -839,9 +935,12 @@ func (h *Handler) createSamples(cfg *v1.Config, updateIfPresent, registryChanged
 
 	}
 	for _, imagestream := range imagestreams {
-		is, err := h.imageclientwrapper.Get("openshift", imagestream.Name, metav1.GetOptions{})
+		if h.abortForDelete(cfg) {
+			return true, nil
+		}
+		is, err := h.imageclientwrapper.Get(imagestream.Name)
 		if err != nil && !kerrors.IsNotFound(err) {
-			return err
+			return false, err
 		}
 
 		if err == nil && !updateIfPresent {
@@ -857,20 +956,23 @@ func (h *Handler) createSamples(cfg *v1.Config, updateIfPresent, registryChanged
 			if updateIfPresent {
 				cache.RemoveUpsert(imagestream.Name)
 			}
-			return err
+			return false, err
 		}
 	}
 
 	// if after initial startup, and not migration, the only cfg change was changing the registry, since that does not impact
 	// the templates, we can move on
 	if len(unskippedTemplates) == 0 && registryChanged && cfg.ConditionTrue(v1.SamplesExist) && cfg.ConditionFalse(v1.MigrationInProgress) {
-		return nil
+		return false, nil
 	}
 
 	for _, fileName := range h.templateFile {
+		if h.abortForDelete(cfg) {
+			return true, nil
+		}
 		template, err := h.Filetemplategetter.Get(fileName)
 		if err != nil {
-			return err
+			return false, err
 		}
 
 		// if unskippedTemplates has >0 entries, then we are down this path to only upsert the templates
@@ -881,9 +983,9 @@ func (h *Handler) createSamples(cfg *v1.Config, updateIfPresent, registryChanged
 			}
 		}
 
-		t, err := h.templateclientwrapper.Get("openshift", template.Name, metav1.GetOptions{})
+		t, err := h.templateclientwrapper.Get(template.Name)
 		if err != nil && !kerrors.IsNotFound(err) {
-			return err
+			return false, err
 		}
 
 		if err == nil && !updateIfPresent {
@@ -896,10 +998,21 @@ func (h *Handler) createSamples(cfg *v1.Config, updateIfPresent, registryChanged
 
 		err = h.upsertTemplate(template, t, cfg)
 		if err != nil {
-			return err
+			return false, err
 		}
 	}
-	return nil
+	return false, nil
+}
+
+// refetchCfgMinimizeConflicts is a best effort attempt to get a later version of the config object
+// just prior to updating the status; if there are any issues retrieving (unlikely since we are accessing
+// the informer's lister) we'll just use the current copy we got from the event or prior fetch
+func (h *Handler) refetchCfgMinimizeConflicts(cfg *v1.Config) *v1.Config {
+	c, e := h.crdwrapper.Get(cfg.Name)
+	if e == nil {
+		return c
+	}
+	return cfg
 }
 
 func getTemplateClient(restconfig *restclient.Config) (*templatev1client.TemplateV1Client, error) {
