@@ -303,11 +303,11 @@ func TestProcessed(t *testing.T) {
 			t.Fatalf("is did not reach client %s: %#v", key, h)
 		}
 	}
-	is, _ := fakeisclient.Get("", "foo", metav1.GetOptions{})
+	is, _ := fakeisclient.Get("foo")
 	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
 		t.Fatalf("stream repo not updated %#v, %#v", is, h)
 	}
-	is, _ = fakeisclient.Get("", "bar", metav1.GetOptions{})
+	is, _ = fakeisclient.Get("bar")
 	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
 		t.Fatalf("stream repo not updated %#v, %#v", is, h)
 	}
@@ -334,11 +334,11 @@ func TestProcessed(t *testing.T) {
 	validate(true, err, "", cfg,
 		conditions,
 		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
-	is, _ = fakeisclient.Get("", "foo", metav1.GetOptions{})
+	is, _ = fakeisclient.Get("foo")
 	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
 		t.Fatalf("stream repo not updated %#v, %#v", is, h)
 	}
-	is, _ = fakeisclient.Get("", "bar", metav1.GetOptions{})
+	is, _ = fakeisclient.Get("bar")
 	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
 		t.Fatalf("stream repo not updated %#v, %#v", is, h)
 	}
@@ -796,6 +796,7 @@ func TestImageStreamImportError(t *testing.T) {
 			},
 		},
 	}
+	turnBackThreeHours := true
 	for _, is := range streams {
 		h, cfg, _ := setup()
 		mimic(&h, x86OCPContentRootDir)
@@ -806,6 +807,9 @@ func TestImageStreamImportError(t *testing.T) {
 		progressing.Status = corev1.ConditionTrue
 		progressing.Reason = is.Name + " "
 		cfg.ConditionUpdate(progressing)
+		needCreds := cfg.Condition(v1.ImportCredentialsExist)
+		needCreds.Status = corev1.ConditionTrue
+		cfg.ConditionUpdate(needCreds)
 		err := h.processImageStreamWatchEvent(is, false)
 		if err != nil {
 			t.Fatalf("processImageStreamWatchEvent error %#v for stream %#v", err, is)
@@ -818,6 +822,20 @@ func TestImageStreamImportError(t *testing.T) {
 		if len(importErr.Reason) == 0 || !cfg.NameInReason(importErr.Reason, is.Name) {
 			t.Fatalf("processImageStreamWatchEvent did not set import error reason field %#v for stream %#v", cfg, is)
 		}
+		if turnBackThreeHours {
+			importErr.LastTransitionTime.Time = metav1.Now().Add(-3 * time.Hour)
+		} else {
+			importErr.LastTransitionTime = metav1.Now()
+		}
+		cfg.ConditionUpdate(importErr)
+		status, reason, detail := cfg.ClusterOperatorStatusDegradedCondition()
+		if (status != configv1.ConditionTrue && turnBackThreeHours) || (status == configv1.ConditionTrue && !turnBackThreeHours) {
+			t.Fatalf("image import error from %#v not reflected in cluster status %#v", is, cfg)
+		}
+		if status == configv1.ConditionFalse && turnBackThreeHours && (len(reason) == 0 || len(detail) == 0) {
+			t.Fatalf("image import error from is %#v with cfg %#v had reason %s and detail %s", is, cfg, reason, detail)
+		}
+		turnBackThreeHours = !turnBackThreeHours
 	}
 }
 
@@ -1307,7 +1325,7 @@ type fakeImageStreamClientWrapper struct {
 	listerrors   map[string]error
 }
 
-func (f *fakeImageStreamClientWrapper) Get(namespace, name string, opts metav1.GetOptions) (*imagev1.ImageStream, error) {
+func (f *fakeImageStreamClientWrapper) Get(name string) (*imagev1.ImageStream, error) {
 	err, _ := f.geterrors[name]
 	if err != nil {
 		return nil, err
@@ -1316,8 +1334,8 @@ func (f *fakeImageStreamClientWrapper) Get(namespace, name string, opts metav1.G
 	return is, nil
 }
 
-func (f *fakeImageStreamClientWrapper) List(namespace string, opts metav1.ListOptions) (*imagev1.ImageStreamList, error) {
-	err, _ := f.geterrors[namespace]
+func (f *fakeImageStreamClientWrapper) List(opts metav1.ListOptions) (*imagev1.ImageStreamList, error) {
+	err, _ := f.geterrors["openshift"]
 	if err != nil {
 		return nil, err
 	}
@@ -1328,7 +1346,7 @@ func (f *fakeImageStreamClientWrapper) List(namespace string, opts metav1.ListOp
 	return list, nil
 }
 
-func (f *fakeImageStreamClientWrapper) Create(namespace string, stream *imagev1.ImageStream) (*imagev1.ImageStream, error) {
+func (f *fakeImageStreamClientWrapper) Create(stream *imagev1.ImageStream) (*imagev1.ImageStream, error) {
 	if stream == nil {
 		return nil, nil
 	}
@@ -1341,15 +1359,15 @@ func (f *fakeImageStreamClientWrapper) Create(namespace string, stream *imagev1.
 	return stream, nil
 }
 
-func (f *fakeImageStreamClientWrapper) Update(namespace string, stream *imagev1.ImageStream) (*imagev1.ImageStream, error) {
-	return f.Create(namespace, stream)
+func (f *fakeImageStreamClientWrapper) Update(stream *imagev1.ImageStream) (*imagev1.ImageStream, error) {
+	return f.Create(stream)
 }
 
-func (f *fakeImageStreamClientWrapper) Delete(namespace, name string, opts *metav1.DeleteOptions) error {
+func (f *fakeImageStreamClientWrapper) Delete(name string, opts *metav1.DeleteOptions) error {
 	return nil
 }
 
-func (f *fakeImageStreamClientWrapper) Watch(namespace string) (watch.Interface, error) {
+func (f *fakeImageStreamClientWrapper) Watch() (watch.Interface, error) {
 	return nil, nil
 }
 
@@ -1361,7 +1379,7 @@ type fakeTemplateClientWrapper struct {
 	listerrors   map[string]error
 }
 
-func (f *fakeTemplateClientWrapper) Get(namespace, name string, opts metav1.GetOptions) (*templatev1.Template, error) {
+func (f *fakeTemplateClientWrapper) Get(name string) (*templatev1.Template, error) {
 	err, _ := f.geterrors[name]
 	if err != nil {
 		return nil, err
@@ -1370,8 +1388,8 @@ func (f *fakeTemplateClientWrapper) Get(namespace, name string, opts metav1.GetO
 	return is, nil
 }
 
-func (f *fakeTemplateClientWrapper) List(namespace string, opts metav1.ListOptions) (*templatev1.TemplateList, error) {
-	err, _ := f.geterrors[namespace]
+func (f *fakeTemplateClientWrapper) List(opts metav1.ListOptions) (*templatev1.TemplateList, error) {
+	err, _ := f.geterrors["openshift"]
 	if err != nil {
 		return nil, err
 	}
@@ -1382,7 +1400,7 @@ func (f *fakeTemplateClientWrapper) List(namespace string, opts metav1.ListOptio
 	return list, nil
 }
 
-func (f *fakeTemplateClientWrapper) Create(namespace string, t *templatev1.Template) (*templatev1.Template, error) {
+func (f *fakeTemplateClientWrapper) Create(t *templatev1.Template) (*templatev1.Template, error) {
 	if t == nil {
 		return nil, nil
 	}
@@ -1395,15 +1413,15 @@ func (f *fakeTemplateClientWrapper) Create(namespace string, t *templatev1.Templ
 	return t, nil
 }
 
-func (f *fakeTemplateClientWrapper) Update(namespace string, t *templatev1.Template) (*templatev1.Template, error) {
-	return f.Create(namespace, t)
+func (f *fakeTemplateClientWrapper) Update(t *templatev1.Template) (*templatev1.Template, error) {
+	return f.Create(t)
 }
 
-func (f *fakeTemplateClientWrapper) Delete(namespace, name string, opts *metav1.DeleteOptions) error {
+func (f *fakeTemplateClientWrapper) Delete(name string, opts *metav1.DeleteOptions) error {
 	return nil
 }
 
-func (f *fakeTemplateClientWrapper) Watch(namespace string) (watch.Interface, error) {
+func (f *fakeTemplateClientWrapper) Watch() (watch.Interface, error) {
 	return nil, nil
 }
 
@@ -1445,7 +1463,7 @@ type fakeCRDWrapper struct {
 	cfg       *v1.Config
 }
 
-func (f *fakeCRDWrapper) UpdateStatus(opcfg *v1.Config) error { return f.updateerr }
+func (f *fakeCRDWrapper) UpdateStatus(opcfg *v1.Config, dbg string) error { return f.updateerr }
 
 func (f *fakeCRDWrapper) Update(opcfg *v1.Config) error { return f.updateerr }
 
