@@ -14,8 +14,6 @@ import (
 )
 
 func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted bool) error {
-	// our pattern is the top most caller locks the mutex
-
 	cfg, filePath, doUpsert, err := h.prepSamplesWatchEvent("imagestream", is.Name, is.Annotations, deleted)
 	if cfg != nil && cfg.ConditionTrue(v1.ImageChangesInProgress) {
 		logrus.Printf("Imagestream %s watch event do upsert %v; no errors in prep %v,  possibly update operator conditions %v", is.Name, doUpsert, err == nil, cfg != nil)
@@ -33,6 +31,7 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 		// if we initiated the change, it will be in our cache, and we want to process
 		// the in progress, import error conditions
 		anyChange := false
+		nonMatchDetail := ""
 		if cache.UpsertsAmount() > 0 {
 			cache.AddReceivedEventFromUpsert(is)
 			if !cfg.ConditionTrue(v1.ImageChangesInProgress) || !cfg.ConditionTrue(v1.SamplesExist) {
@@ -45,11 +44,12 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 			}
 			streams := cache.GetUpsertImageStreams()
 			keysToClear := []string{}
+			cfg = h.refetchCfgMinimizeConflicts(cfg)
 			for key, is := range streams {
 				if is == nil {
 					// never got update, refetch
 					var e error
-					is, e = h.imageclientwrapper.Get("openshift", key, kapis.GetOptions{})
+					is, e = h.imageclientwrapper.Get(key)
 					if e != nil {
 						keysToClear = append(keysToClear, key)
 						anyChange = true
@@ -57,11 +57,16 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 					}
 				}
 
+				if !cfg.NameInReason(cfg.Condition(v1.ImageChangesInProgress).Reason, is.Name) &&
+					!cfg.NameInReason(cfg.Condition(v1.ImportImageErrorsExist).Reason, is.Name) {
+					logrus.Printf("skipping progress check (caching phase) for %s because it has been removed from the image progress and import error conditions", is.Name)
+					continue
+				}
+
 				// see if we should turn off pending, clear errors
-				nonMatchDetail := ""
 				cfg, nonMatchDetail, anyChange = h.processImportStatus(is, cfg)
-				if !anyChange {
-					logrus.Printf("imagestream %s still not finished with its image imports, including %s", is.Name, nonMatchDetail)
+				if !anyChange && len(nonMatchDetail) > 0 {
+					logrus.Printf("imagestream %s still not finished with its image imports (caching phase), including %s", is.Name, nonMatchDetail)
 					// cached instances are processed at the end of the handler Config event flow
 					return nil
 				}
@@ -75,22 +80,38 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 			// unless we were unable to fetch any of the imagestreams
 			// and if necessary processImportStatus would have set in progress to false as needed;
 			if anyChange {
-				logrus.Printf("CRDUPDATE updating progress/error condition (within caching block) after results for %s", is.Name)
-				err = h.crdwrapper.UpdateStatus(cfg)
-				if err == nil {
-					// only clear out the cache if our update got through
-					cache.ClearUpsertsCache()
-				}
+				dbg := fmt.Sprintf("updating progress/error condition (within caching block) after results for %s", is.Name)
+				logrus.Printf("CRDUPDATE %s", dbg)
+				err = h.crdwrapper.UpdateStatus(cfg, dbg)
+				// we used to not clear the cache until we confirmed the update occurred; but for
+				// whatever reason, we started seeing more conflicts here on the k8s 1.13 rebase;
+				// and it turns out, after the `cache.UpsertsAmount() > 0` check above it is tricky
+				// to discern a retry because of an err here vs. our not having finished upserts yet;
+				// so since we have decided the condition, most likely we are done; so clear the cache
+				// and fall into our steady state process imagestream event individually
+				cache.ClearUpsertsCache()
 			}
 			return err
 		}
+
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
+		if !cfg.NameInReason(cfg.Condition(v1.ImageChangesInProgress).Reason, is.Name) &&
+			!cfg.NameInReason(cfg.Condition(v1.ImportImageErrorsExist).Reason, is.Name) {
+			logrus.Printf("aborting progress check (post-cache) for %s because it has been removed from the image progress and import error conditions", is.Name)
+		}
 		// otherwise, if someone else changed it in such a way that we don't want to
-		// upsert it again, or we missed an event and this is a relist, update conditions
+		// upsert it again, or we missed an event and this is a relist, or we had an update conflict
+		// after completing cache process, update conditions as needed
 		// as needed
-		cfg, _, anyChange = h.processImportStatus(is, cfg)
+		cfg, nonMatchDetail, anyChange = h.processImportStatus(is, cfg)
 		if anyChange {
-			logrus.Printf("CRDUPDATE updating progress/error condition after results for %s", is.Name)
-			return h.crdwrapper.UpdateStatus(cfg)
+			dbg := fmt.Sprintf("updating progress/error condition after results for %s", is.Name)
+			logrus.Printf("CRDUPDATE %s", dbg)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
+		} else {
+			if len(nonMatchDetail) > 0 {
+				logrus.Printf("imagestream %s still not finished with its image imports, including %s", is.Name, nonMatchDetail)
+			}
 		}
 		return nil
 
@@ -99,7 +120,9 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 	// prepWatchEvent has determined we actually need to do the upsert
 
 	if cfg == nil {
-		return fmt.Errorf("cannot upsert imagestream %s because could not obtain Config", is.Name)
+		// prepSamplesWatch will handle logging here as needed, as well as provide the appropriate
+		// non-nil or nil setting for err
+		return err
 	}
 
 	if cfg.ClusterNeedsCreds() {
@@ -109,9 +132,11 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 	imagestream, err := h.Fileimagegetter.Get(filePath)
 	if err != nil {
 		// still attempt to report error in status
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
 		h.processError(cfg, v1.SamplesExist, corev1.ConditionUnknown, err, "%v error reading file %s", filePath)
-		logrus.Printf("CRDUPDATE event img update err bad fs read %s", filePath)
-		h.crdwrapper.UpdateStatus(cfg)
+		dbg := fmt.Sprintf("event img update err bad fs read %s", filePath)
+		logrus.Printf("CRDUPDATE %s", dbg)
+		h.crdwrapper.UpdateStatus(cfg, dbg)
 		// if we get this, don't bother retrying
 		return nil
 	}
@@ -129,12 +154,14 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 			// it the main loop created, we will let it set the image change reason field
 			return nil
 		}
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
 		h.processError(cfg, v1.SamplesExist, corev1.ConditionUnknown, err, "%v error replacing imagestream %s", imagestream.Name)
-		logrus.Printf("CRDUPDATE event img update err bad api obj update %s", imagestream.Name)
-		h.crdwrapper.UpdateStatus(cfg)
-		return err
+		dbg := fmt.Sprintf("CRDUPDATE event img update err bad api obj update %s", imagestream.Name)
+		logrus.Printf("CRDUPDATE %s", dbg)
+		return h.crdwrapper.UpdateStatus(cfg, dbg)
 	}
 	// now update progressing condition
+	cfg = h.refetchCfgMinimizeConflicts(cfg)
 	progressing := cfg.Condition(v1.ImageChangesInProgress)
 	now := kapis.Now()
 	progressing.LastUpdateTime = now
@@ -145,8 +172,9 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 		progressing.Reason = progressing.Reason + imagestream.Name + " "
 	}
 	cfg.ConditionUpdate(progressing)
-	logrus.Printf("CRDUPDATE progressing true update for imagestream %s", imagestream.Name)
-	return h.crdwrapper.UpdateStatus(cfg)
+	dbg := fmt.Sprintf("progressing true update for imagestream %s", imagestream.Name)
+	logrus.Printf("CRDUPDATE %s", dbg)
+	return h.crdwrapper.UpdateStatus(cfg, dbg)
 
 }
 
@@ -165,14 +193,14 @@ func (h *Handler) upsertImageStream(imagestreamInOperatorImage, imagestreamInClu
 				imagestreamInCluster.Labels = make(map[string]string)
 			}
 			imagestreamInCluster.Labels[v1.SamplesManagedLabel] = "false"
-			h.imageclientwrapper.Update("openshift", imagestreamInCluster)
+			h.imageclientwrapper.Update(imagestreamInCluster)
 			// if we get an error, we'll just try to remove the label next
 			// time; and we'll examine the skipped lists on delete
 		}
 		return nil
 	}
 
-	h.updateDockerPullSpec([]string{"docker.io", "registry.redhat.io", "registry.access.redhat.com", "quay.io"}, imagestreamInOperatorImage, opcfg)
+	h.updateDockerPullSpec([]string{"docker.io", "registry.redhat.io", "registry.access.redhat.com", "quay.io", "registry.svc.ci.openshift.org"}, imagestreamInOperatorImage, opcfg)
 
 	if imagestreamInOperatorImage.Labels == nil {
 		imagestreamInOperatorImage.Labels = make(map[string]string)
@@ -184,7 +212,7 @@ func (h *Handler) upsertImageStream(imagestreamInOperatorImage, imagestreamInClu
 	imagestreamInOperatorImage.Annotations[v1.SamplesVersionAnnotation] = h.version
 
 	if imagestreamInCluster == nil {
-		_, err := h.imageclientwrapper.Create("openshift", imagestreamInOperatorImage)
+		_, err := h.imageclientwrapper.Create(imagestreamInOperatorImage)
 		if err != nil {
 			if kerrors.IsAlreadyExists(err) {
 				logrus.Printf("imagestream %s recreated since delete event", imagestreamInOperatorImage.Name)
@@ -197,8 +225,29 @@ func (h *Handler) upsertImageStream(imagestreamInOperatorImage, imagestreamInClu
 		return nil
 	}
 
+	// with the advent of removing EOL images in 4.2, we cannot just update the imagestream
+	// with client-go/Update as that will replace vs. patch; we need to preserve EOL specs
+	// if the imagestream was created during 4.1 as to allow pullthrough on builds for example
+	// so we manually merge the tags in the existing image stream that do not exist in the one
+	// in the payload
+	tagsToAdd := []imagev1.TagReference{}
+	for _, existingTag := range imagestreamInCluster.Spec.Tags {
+		found := false
+		for _, newTag := range imagestreamInOperatorImage.Spec.Tags {
+			if newTag.Name == existingTag.Name {
+				found = true
+				break
+			}
+		}
+		if !found {
+			tagsToAdd = append(tagsToAdd, existingTag)
+		}
+	}
+	for _, tag := range tagsToAdd {
+		imagestreamInOperatorImage.Spec.Tags = append(imagestreamInOperatorImage.Spec.Tags, tag)
+	}
 	imagestreamInOperatorImage.ResourceVersion = imagestreamInCluster.ResourceVersion
-	_, err := h.imageclientwrapper.Update("openshift", imagestreamInOperatorImage)
+	_, err := h.imageclientwrapper.Update(imagestreamInOperatorImage)
 	if err != nil {
 		return h.processError(opcfg, v1.SamplesExist, corev1.ConditionUnknown, err, "imagestream update error: %v")
 	}
@@ -207,16 +256,6 @@ func (h *Handler) upsertImageStream(imagestreamInOperatorImage, imagestreamInClu
 }
 
 func (h *Handler) updateDockerPullSpec(oldies []string, imagestream *imagev1.ImageStream, opcfg *v1.Config) {
-	// we always want to leave the jenkins images as using the payload set to the IMAGE* env's;
-	switch imagestream.Name {
-	case "jenkins":
-		return
-	case "jenkins-agent-nodejs":
-		return
-	case "jenkins-agent-maven":
-		return
-	}
-
 	if len(opcfg.Spec.SamplesRegistry) == 0 {
 		return
 	}
@@ -281,6 +320,7 @@ func (h *Handler) clearStreamFromImportError(name string, importError *v1.Config
 	if cfg.NameInReason(importError.Reason, name) {
 		logrus.Printf("clearing imagestream %s from the import image error condition", name)
 	}
+	oldStatus := importError.Status
 	start := strings.Index(importError.Message, "<imagestream/"+name+">")
 	end := strings.LastIndex(importError.Message, "<imagestream/"+name+">")
 	if start >= 0 && end > 0 {
@@ -295,7 +335,9 @@ func (h *Handler) clearStreamFromImportError(name string, importError *v1.Config
 		} else {
 			importError.Status = corev1.ConditionTrue
 		}
-		importError.LastTransitionTime = now
+		if oldStatus != importError.Status {
+			importError.LastTransitionTime = now
+		}
 		importError.LastUpdateTime = now
 		cfg.ConditionUpdate(importError)
 	}
@@ -373,15 +415,23 @@ func (h *Handler) processImportStatus(is *imagev1.ImageStream, cfg *v1.Config) (
 		}
 
 		if !anyErrors {
-			if len(is.Spec.Tags) == len(is.Status.Tags) {
+			// it is possible after an upgrade that tags can be removed because of EOL processing;
+			// since we do not delete those EOL images from the imagestream status (so as to not break
+			// existing deployments referencing specific tags), it is possible that a valid configuration
+			// has less tags in the spec than the status (vs. an equal amount being the only valid combination)
+			if len(is.Spec.Tags) <= len(is.Status.Tags) {
 				for _, specTag := range is.Spec.Tags {
 					matched := false
+					foundTagInStatus := false
+					foundItemsInStatusTag := false
 					for _, statusTag := range is.Status.Tags {
 						logrus.Debugf("checking spec tag %s against status tag %s with num items %d", specTag.Name, statusTag.Tag, len(statusTag.Items))
 						if specTag.Name == statusTag.Tag {
+							foundTagInStatus = true
 							// if the latest gens have no errors, see if we got gen match
 							if statusTag.Items != nil {
 								for _, event := range statusTag.Items {
+									foundItemsInStatusTag = true
 									if specTag.Generation != nil {
 										logrus.Debugf("checking status tag %d against spec tag %d", event.Generation, *specTag.Generation)
 									}
@@ -398,12 +448,25 @@ func (h *Handler) processImportStatus(is *imagev1.ImageStream, cfg *v1.Config) (
 					}
 					if !matched {
 						pending = true
+						if len(nonMatchDetail) == 0 {
+							switch {
+							case foundTagInStatus == false:
+								nonMatchDetail = fmt.Sprintf("spec tag %s did not have a matching status tag", specTag.Name)
+							case foundItemsInStatusTag == false:
+								nonMatchDetail = fmt.Sprintf("status tag %s had no items in it", specTag.Name)
+							default:
+								nonMatchDetail = fmt.Sprintf("match failed for some unknown reason; spec: %#v status: %#v", is.Spec.Tags, is.Status.Tags)
+							}
+						}
 						break
 					}
 				}
+				if len(is.Spec.Tags) == 0 {
+					nonMatchDetail = fmt.Sprintf("there were no spec tags")
+				}
 			} else {
 				pending = true
-				nonMatchDetail = "the number of status tags does not equal the number of spec tags"
+				nonMatchDetail = "the number of status tags is less than the number of spec tags"
 			}
 		}
 	} else {
@@ -421,6 +484,7 @@ func (h *Handler) processImportStatus(is *imagev1.ImageStream, cfg *v1.Config) (
 		if !anyErrors {
 			h.clearStreamFromImportError(is.Name, importError, cfg)
 		}
+		logrus.Printf("There are no more image imports in flight for imagestream %s", is.Name)
 		if processing.Status == corev1.ConditionTrue {
 			now := kapis.Now()
 			// remove this imagestream name, including the space separator, from processing
@@ -437,7 +501,6 @@ func (h *Handler) processImportStatus(is *imagev1.ImageStream, cfg *v1.Config) (
 				processing.LastTransitionTime = now
 				processing.LastUpdateTime = now
 				cfg.ConditionUpdate(processing)
-				logrus.Printf("There are no more image imports in flight for imagestream %s", is.Name)
 				anyConditionUpdate = true
 			}
 		}
